import Flutter
import UIKit
import JTProximitySDK

public class SwiftFlutterJointagPlugin: NSObject, FlutterPlugin {
    public static func initializeWithOptions(launchOptions: [UIApplication.LaunchOptionsKey: Any]?, apiKey: String, apiSecret: String) {
        ProximitySDK.shared.setLogLevel(JTPLogLevel.verbose)
        ProximitySDK.shared.initWithLaunchOptions(launchOptions, apiKey: apiKey, apiSecret: apiSecret)
    }
    
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "flutter_jointag", binaryMessenger: registrar.messenger())
    let instance = SwiftFlutterJointagPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
    registrar.addApplicationDelegate(instance)
    
  }

    public func applicationDidBecomeActive(_ application: UIApplication) {
        debugPrint("applicationDidBecomeActive")
    }

    public func applicationWillTerminate(_ application: UIApplication) {
        debugPrint("applicationWillTerminate")
    }

    public func applicationWillResignActive(_ application: UIApplication) {
        debugPrint("applicationWillResignActive")
    }

    public func applicationDidEnterBackground(_ application: UIApplication) {
        debugPrint("applicationDidEnterBackground")
    }

    public func applicationWillEnterForeground(_ application: UIApplication) {
        print("applicationWillEnterForeground")
    }
    
    public func application(_ application: UIApplication, didReceive notification: UILocalNotification) {
        debugPrint("notification")
        
        if #available(iOS 10.0, *) {
            if (ProximitySDK.shared.application(application, didReceive: notification)){
                return;
            }
        } else {
            // Fallback on earlier versions
        }
    }
    @available(iOS 10.0, *)
    public func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        if ProximitySDK.shared.userNotificationCenter(center, didReceive: response) {
            completionHandler()
            return
        }
    }
    @available(iOS 10.0, *)
    public func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        if ProximitySDK.shared.userNotificationCenter(center, willPresent: notification) {
            completionHandler([.alert, .badge, .sound])
            return
        }
    }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
        case "checkPendingPermissions":
            break;
        case  "requestTrackingAuthorization":
            ProximitySDK.shared.requestTrackingAuthorization()
            result(true);
            break;
        case "requestNotificationAuthorization":
            ProximitySDK.shared.requestNotificationAuthorization()
            result(true);
            break;
        case "requestLocationAuthorization":
            ProximitySDK.shared.requestLocationAuthorization()
            result(true);
            break;
        case "getInstallationId":
            result(ProximitySDK.shared.installationId())
            break;
        case "setAdvertisingEnabled":
            ProximitySDK.shared.advertisingEnabled=true;
            result(ProximitySDK.shared.advertisingEnabled)
            break;
        case "setAdvertisingDisabled":
            ProximitySDK.shared.advertisingEnabled=false;
            result(ProximitySDK.shared.advertisingEnabled)
            break;
        case "enabledCmp":
            ProximitySDK.shared.cmpEnabled.toggle();
            result(ProximitySDK.shared.cmpEnabled);
            break;
        case "setBaseJobId":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "setBaseJobId for iOS Not Implemented",
                                details: nil));
            break;
        case "getAdvertisingIdentifier":
            result(ProximitySDK.shared.advertisingId());
            break;
        case "getApiKey":
            result(ProximitySDK.shared.apiKey);
            break;
        case "getContext":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "getContext for iOS Not Implemented",
                                details: nil));
            break;
        case "getGDPRConsent":
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any],
                let vendor = myArgs["VENDOR_ID"] as? Int32{
                result(ProximitySDK.shared.getGDPRConsent(forVendor: vendor));
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                   "flutter arguments in method: (getGDPRConsent)", details: nil))
            }
            break;
        case "getSecret":
            result(ProximitySDK.shared.apiSecret);
            break;
        case "getSessionId":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "getSessionId for iOS Not Implemented",
                                details: nil));
            break;
        case "isLimitAdTrackingEnabled":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "isLimitAdTrackingEnabled for iOS Not Implemented",
                                details: nil));
            break;
        //da fare
        case "getCustomActionListeners":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "getCustomActionListeners for iOS Not Implemented",
                                details: nil));
            break;
        case "addCustomActionListener":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "addCustomActionListener for iOS Not Implemented",
                                details: nil));
            break;
        case "removeCustomActionListener":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "removeCustomActionListener for iOS Not Implemented",
                                details: nil));
            break;

        case "getAdvertisingEnabled":
            result(ProximitySDK.shared.advertisingEnabled)
            break;
        case "setGDPRConsent":
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any],
                let vendor = myArgs["vendor"] as? Int32,
                let consent = myArgs["consent"] as? Bool{
                ProximitySDK.shared.setGDPRConsent(consent, forVendor: vendor)
                result(ProximitySDK.shared.getGDPRConsent(forVendor: vendor));
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                   "flutter arguments in method: (setGDPRConsent)", details: nil))
            }
            break;
        case "setInstallationId":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "setInstallationId for iOS Not Implemented",
                                details: nil));
            break;
        case "setSessionId":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "setSessionId for iOS Not Implemented",
                                details: nil));
            break;
        default:
            result(FlutterError(code: "UNAVAILABLE",
                                message: "Unknown Method for iOS Not Implemented",
                                details: nil));
            break;
    }
  }
}
