import 'dart:async';
import 'dart:developer';
import 'package:flutter/services.dart';
import 'custom_action_listeners.dart';

class FlutterJointag {
  static const MethodChannel _channel = const MethodChannel('flutter_jointag');
  static String API_KEY;
  static String SECRET;
  static int NEWJOB_ID;
  static String INSTALLATION_ID;
  static String SESSION_ID;
  static int VENDOR_ID;
  static bool CONSENT;

  static Future get Init async {
    log("CALLED: Init");
    try {
      await _channel.invokeMethod("Init", {
        'API_KEY': API_KEY,
        'SECRET': SECRET,
      });
      return;
    } on PlatformException catch (e) {
      throw e;
    }
  }
  static Future get requestTrackingAuthorization async {
    log("CALLED: requestTrackingAuthorization");
    try {
      await _channel.invokeMethod('requestTrackingAuthorization');
      return;
    } on PlatformException catch (e) {
      throw e;
    }
  }
  static Future get requestNotificationAuthorization async {
    log("CALLED: requestNotificationAuthorization");
    try {
      await _channel.invokeMethod('requestNotificationAuthorization');
      return;
    } on PlatformException catch (e) {
      throw e;
    }
  }
  static Future get requestLocationAuthorization async {
    log("CALLED: requestLocationAuthorization");
    try {
      await _channel.invokeMethod('requestLocationAuthorization');
      return;
    } on PlatformException catch (e) {
      throw e;
    }
  }
  static Future get checkPendingPermissions async {
    log("CALLED: checkPendingPermissions");
    try {
      await _channel.invokeMethod('checkPendingPermissions');
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get getAdvertisingEnabled async {
    log("CALLED: getAdvertisingEnabled");
    try {
      final bool advertisingEnabled =
      await _channel.invokeMethod('getAdvertisingEnabled');
      log(advertisingEnabled.toString());
      return advertisingEnabled;
    } on PlatformException catch (e) {
      throw e;
    }
  }
  
  static Future<String> get getInstallationId async {
    log("CALLED: getInstallationId");
    try {
      final String installationId =
          await _channel.invokeMethod('getInstallationId');
      log(installationId);
      return installationId;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get setAdvertisingDisabled async {
    log("CALLED: setAdvertisingDisabled");
    try {
      await _channel.invokeMethod('setAdvertisingDisabled');
      return true;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get setAdvertisingEnabled async {
    log("CALLED: setAdvertisingEnabled");
    try {
      await _channel.invokeMethod('setAdvertisingEnabled');
      return true;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get enabledCmp async {
    log("CALLED: enabledCmp");
    try {
      await _channel.invokeMethod('enabledCmp');
      return true;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get setGDPRConsent async {
    log("CALLED: setGDPRConsent");
    try {
      int vendorId = 12;
      bool consent = true;
      await _channel.invokeMethod('setGDPRConsent', {'vendor': vendorId, 'consent' : consent});
      return true;
    } on PlatformException catch (e) {
      throw e;
    }
  }
  
  static Future<bool> get setBaseJobId async {
    log("CALLED: setBaseJobId");
    try {
      await _channel.invokeMethod('setBaseJobId', {'newJobID': NEWJOB_ID});
      return true;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get setInstallationId async {
    log("CALLED: setInstallationId");
    try {
      await _channel.invokeMethod('setInstallationId', {'InstallationId': INSTALLATION_ID});
      return true;
    } on PlatformException catch (e) {
      throw e;
    }
  }
  static Future<bool> get setSessionId async {
    log("CALLED: setSessionId");
    try {
      await _channel.invokeMethod('setSessionId', {'SessionId': SESSION_ID});
      return true;
    } on PlatformException catch (e) {
      throw e;
    }
  }
  static Future<String> get getAdvertisingIdentifier async {
    log("CALLED: getAdvertisingIdentifier");
    try {
      final String advertisingIdentifier =
          await _channel.invokeMethod('getAdvertisingIdentifier');
      log(advertisingIdentifier);
      return advertisingIdentifier;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<String> get getApiKey async {
    log("CALLED: getApiKey");
    try {
      final String apiKey = await _channel.invokeMethod('getApiKey');
      log(apiKey);
      return apiKey;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<String> get getContext async {
    log("CALLED: getContext");
    try {
      final String context = await _channel.invokeMethod('getContext');
      log(context);
      return context;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get getGDPRConsent async {
    log("CALLED: getGDPRConsent");
    try {
      int vendorId = 12;
      final bool gDPRConsent = await _channel.invokeMethod('getGDPRConsent', {
        'VENDOR_ID': vendorId,
      });
      log(gDPRConsent.toString());
      return gDPRConsent;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<String> get getSecret async {
    log("CALLED: getSecret");
    try {
      final String secret = await _channel.invokeMethod('getSecret');
      log(secret);
      return secret;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<String> get getSessionId async {
    log("CALLED: getSessionId");
    try {
      final String sessionId = await _channel.invokeMethod('getSessionId');
      log(sessionId);
      return sessionId;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get isLimitAdTrackingEnabled async {
    log("CALLED: isLimitAdTrackingEnabled");
    try {
      final bool limitAdTrackingEnabled =
          await _channel.invokeMethod('isLimitAdTrackingEnabled');
      log(limitAdTrackingEnabled.toString());
      return limitAdTrackingEnabled;
    } on PlatformException catch (e) {
      throw e;
    }
  }


//todo terminare gestione listeners
  static Future<List<dynamic>> get getCustomActionListeners async {
    log("CALLED: getCustomActionListeners");
    try {
      final dynamic customActionListeners =
      await _channel.invokeMethod('getCustomActionListeners');
      log(customActionListeners.join(","));
      return customActionListeners;
    } on PlatformException catch (e) {
      throw e;
    }
  }

  static Future<bool> get addCustomActionListeners async {
    log("CALLED: addCustomActionListeners");
    try {
      var cancel = startListening((payload) {
        log(payload);
      });
      await Future.delayed(Duration(seconds: 4));
      cancel();
      //await _channel.invokeMethod('addCustomActionListeners', {'newJobID': NEWJOBID});
      //log("ecco");
      return true;
    } on PlatformException catch (e) {
      throw e;
    }
  }
}
